/*
Write-once filesystem

Copyright (c) 2022 Aron Barath
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the Author nor the names of contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

Source code: svn://svn.repo.hu/wroncefs/trunk
Contact the author: aron-dev@mailbox.org
*/

#ifndef IMPL_H
#define IMPL_H

#include <stddef.h>
#include <stdint.h>

#define W_SEEK_SET  0
#define W_SEEK_CUR  1
#define W_SEEK_END  2

struct w_dir;
struct w_file;

struct w_dirent
{
	size_t ino;
	size_t namelen;
	char*  name;
};

struct w_stat
{
	size_t is_file;
	size_t size;
};

struct w_statfs
{
	size_t bsize;
	size_t bfree;
	size_t blocks;
	size_t namelen;
};

struct impl
{
	const char* impl_name;
	uint32_t    max_filesize;
	uint32_t    min_blockszsh;
	uint32_t    max_blockszsh;
	int         inf_can_alter;
	int         inf_16bit_mode;

	int (*f_off2shift)(uint32_t size);
	const char* (*f_strerror)(int err);

	int (*f_mount)(uint16_t nblocks, uint8_t blockszsh);
	int (*f_fsck)(void);
	const char* (*f_getcwd)(void);
	int (*f_chdir)(const char* name);
	int (*f_mkdir)(const char* name);
	int (*f_rmdir)(const char* name);
	int (*f_unlink)(const char* name);
	int (*f_statfs)(struct w_statfs* const st);
	int (*f_istat)(size_t ino, struct w_stat* const st);

	struct w_dir* (*f_opendir)(const char* name, int* const out_ec);
	struct w_dirent* (*f_readdir)(struct w_dir* const dir, int* const out_ec);
	void (*f_closedir)(struct w_dir* const dir);

	struct w_file* (*f_open)(const char* name, int* const out_ec);
	struct w_file* (*f_creat)(const char* name, int* const out_ec);
	int (*f_read)(struct w_file* const fd, void* buf, int size,
		int* const out_ec);
	int (*f_write)(struct w_file* const fd, const void* buf, int size,
		int* const out_ec);
	int (*f_seek)(struct w_file* const fd, int offs, int whence);
	void (*f_close)(struct w_file* const fd);
};

extern const struct impl impl_alter16;
extern const struct impl impl_alter32;
extern const struct impl impl_noalter16;
extern const struct impl impl_noalter32;

int8_t imgtool_op_write(const void* buf, uint32_t addr, uint32_t size,
	uint8_t blockszsh);
int8_t imgtool_op_erase(uint32_t addr, uint32_t size, uint8_t blockszsh);

#endif /* IMPL_H */
