/*
Write-once filesystem

Copyright (c) 2022 Aron Barath
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the Author nor the names of contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

Source code: svn://svn.repo.hu/wroncefs/trunk
Contact the author: aron-dev@mailbox.org
*/

#include "impl.h"
#include <stddef.h>

struct full_segment
{
	uint8_t data[32];
};

WOFS_IMPL int8_t wofs__copy_storage_content(WOFS_STATE_ARG wofs_addr_t dst,
	wofs_addr_t src, wofs_size_t size)
{
	/* this is a tricky one: if WRONCEFS_DIRECT_READ=1, the loop will be */
	/* much faster than the last unaligned segment copy */

	while(sizeof(struct full_segment)<=size)
	{
		WOFS_READ_LOCAL(struct full_segment, segment, src);

		if(0!=wroncefs_op_write(PASS_STATE WOFS_LOCAL_ADDR(segment), dst,
			sizeof(struct full_segment)))
		{
			return -WOFS_ERR_EIOW;
		}

		dst  += sizeof(struct full_segment);
		src  += sizeof(struct full_segment);
		size -= sizeof(struct full_segment);
	}

	if(0<size)
	{
		uint8_t buf[sizeof(struct full_segment)];

		WOFS_READ_EXTERN(buf, src, size);

		if(0!=wroncefs_op_write(PASS_STATE buf, dst, size))
		{
			return -WOFS_ERR_EIOW;
		}
	}

	return 0;
}
