/*
Write-once filesystem

Copyright (c) 2022 Aron Barath
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the Author nor the names of contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

Source code: svn://svn.repo.hu/wroncefs/trunk
Contact the author: aron-dev@mailbox.org
*/

#include "impl.h"
#include <stddef.h>

WOFS_IMPL int8_t wofs__follow_file_chain__forward(WOFS_STATE_ARG
	struct wofs_fd* const fd)
{
	WOFS_READ_LOCAL(struct wofs_inode, inode, fd->fd_addr);

	const wofs_ino_t next = WOFS_LOCAL_FIELD(inode, i_next);

	if(WOFS_END_INODE==next)
	{
		return -WOFS_ERR_ENOMORE;
	}

	if(0x00!=WOFS_LOCAL_FIELD(inode, i_vnext))
	{
		/* 'i_next' cannot be trusted -- fix this inode by dropping the */
		/* chaining, and possibly leaving some garbage behind that */
		/* subsequent mount could reclaim */

		const int8_t ec = wofs__file_rewrite(PASS_STATE fd, fd->fd_clen);

		if(0!=ec)
		{
			return ec;
		}

		/* the 'i_next' has been just erased, we have no following block */

		return -WOFS_ERR_ENOMORE;
	}

	return wofs__fd_setup(PASS_STATE fd, next);
}

WOFS_IMPL int8_t wofs__follow_file_chain__backward(WOFS_STATE_ARG
	struct wofs_fd* const fd)
{
	WOFS_READ_LOCAL(wofs_ino_t, prev_ino,
		fd->fd_addr+sizeof(struct wofs_inode)+
		offsetof(struct wofs_fileinfo, prev_ino));

	const wofs_ino_t prev = WOFS_LOCAL_VALUE(prev_ino);

	if(WOFS_END_INODE==prev)
	{
		return -WOFS_ERR_ENOTRECOVERABLE;
	}

	const int8_t ec = wofs__fd_setup(PASS_STATE fd, prev);

	if(0!=ec)
	{
		return ec;
	}

	fd->fd_lpos = fd->fd_clen;
	fd->fd_fpos += fd->fd_clen;

	return 0;
}
